@@ -13,6 +13,7 @@
 
 #include <mono/utils/mono-publib.h>
 
+#include <mono/utils/mono-forward.h>
 #include <mono/metadata/object.h>
 #include <mono/metadata/reflection.h>
 
@@ -23,7 +24,6 @@ typedef void (*MonoThreadStartCB) (intptr_t tid, void* stack_start,
 typedef void (*MonoThreadAttachCB) (intptr_t tid, void* stack_start);
 
 typedef struct _MonoAppDomain MonoAppDomain;
-typedef struct _MonoJitInfo MonoJitInfo;
 
 typedef void (*MonoDomainFunc) (MonoDomain *domain, void* user_data);
 
@@ -39,8 +39,7 @@ mono_init_version          (const char *domain_name, const char *version);
 MONO_API MonoDomain*
 mono_get_root_domain       (void);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API void
+MONO_API MONO_RT_EXTERNAL_ONLY void
 mono_runtime_init          (MonoDomain *domain, MonoThreadStartCB start_cb,
 			    MonoThreadAttachCB attach_cb);
 
@@ -65,12 +64,10 @@ mono_check_corlib_version (void);
 MONO_API MonoDomain *
 mono_domain_create         (void);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API MonoDomain *
+MONO_API MONO_RT_EXTERNAL_ONLY MonoDomain *
 mono_domain_create_appdomain (char *friendly_name, char *configuration_file);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API void
+MONO_API MONO_RT_EXTERNAL_ONLY void
 mono_domain_set_config (MonoDomain *domain, const char *base_dir, const char *config_file_name);
 
 MONO_API MonoDomain *
@@ -85,14 +82,13 @@ mono_domain_get_id         (MonoDomain *domain);
 MONO_API const char *
 mono_domain_get_friendly_name (MonoDomain *domain);
 
-MONO_API mono_bool
+MONO_API MONO_RT_EXTERNAL_ONLY mono_bool
 mono_domain_set            (MonoDomain *domain, mono_bool force);
 
-MONO_API void
+MONO_API MONO_RT_EXTERNAL_ONLY void
 mono_domain_set_internal   (MonoDomain *domain);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API void
+MONO_API MONO_RT_EXTERNAL_ONLY void
 mono_domain_unload (MonoDomain *domain);
 
 MONO_API void
@@ -101,8 +97,7 @@ mono_domain_try_unload (MonoDomain *domain, MonoObject **exc);
 MONO_API mono_bool
 mono_domain_is_unloading   (MonoDomain *domain);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API MonoDomain *
+MONO_API MONO_RT_EXTERNAL_ONLY MonoDomain *
 mono_domain_from_appdomain (MonoAppDomain *appdomain);
 
 MONO_API void
@@ -120,19 +115,16 @@ mono_domain_free           (MonoDomain *domain, mono_bool force);
 MONO_API mono_bool
 mono_domain_has_type_resolve (MonoDomain *domain);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API MonoReflectionAssembly *
+MONO_API MONO_RT_EXTERNAL_ONLY MonoReflectionAssembly *
 mono_domain_try_type_resolve (MonoDomain *domain, char *name, MonoObject *tb);
 
 MONO_API mono_bool
 mono_domain_owns_vtable_slot (MonoDomain *domain, void* vtable_slot);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API void
+MONO_API MONO_RT_EXTERNAL_ONLY void
 mono_context_init 				   (MonoDomain *domain);
 
-MONO_RT_EXTERNAL_ONLY
-MONO_API void 
+MONO_API MONO_RT_EXTERNAL_ONLY void
 mono_context_set				   (MonoAppContext *new_context);
 
 MONO_API MonoAppContext * 
@@ -145,7 +137,7 @@ MONO_API int32_t
 mono_context_get_domain_id  (MonoAppContext *context);
 
 MONO_API MonoJitInfo *
-mono_jit_info_table_find   (MonoDomain *domain, char *addr);
+mono_jit_info_table_find   (MonoDomain *domain, void* addr);
 
 /* MonoJitInfo accessors */
 
